import { AtomViewModel } from "./AtomViewModel";
export declare type ILoadOptions = {
    /**
     * Initializes this action as soon as view model has been created.
     * If set true, success will be set to null as default.
     * @default true
     */
    init: true;
    /**
     * Displays an error if error occurs during initialization, to turn it off
     * you can set to false
     * @default true
     */
    showErrorOnInit?: boolean;
    watch?: false;
    watchDelayMS?: never;
} | {
    init?: false;
    showErrorOnInit?: false;
    /**
     * Watch, executes this action whenever any of `this`'s properties updates,
     * this does not refresh when any property is assigned within this method.
     * Any subsequent update is ignored while the async operation is still in process.
     *
     * An automatic delay of 100 milliseconds (you can change this with watchDelayMS)
     * is applied before execution to accumulate
     * all updates and event is executed later on.
     */
    watch: true;
    /**
     * Delay in milliseconds before invoking the watched expression.
     * @default 100
     */
    watchDelayMS?: number;
} | {
    init: true;
    showErrorOnInit?: boolean;
    watch: true;
    watchDelayMS?: number;
} | never;
/**
 * Loads given method on based on init/watch properties.
 * If init is true, method will be executed when view model is initialized.
 * If watch is true, method will be executed when any of `this.*.*` properties are
 * modified. This method can be asynchronous. Watch will ignore all assignment
 * changes within the method.
 *
 * Every execution will be delayed by parameter specified in {@link ILoadOptions#watchDelayMS},
 * so multiple calls can be accumulated and only one final execution will proceed. This is useful
 * when you want to load items from API when user is continuously typing in search box.
 *
 * Method will have an input parameter for cancelToken {@link CancelToken} which you
 * can pass it to any REST Api call, before executing next method, cancelToken will
 * cancel previous execution.
 *
 * Either init or watch has to be true, or both can be true as well.
 */
export default function Load({ init, showErrorOnInit, watch, watchDelayMS }: ILoadOptions): (target: AtomViewModel, key: string | symbol) => void;
//# sourceMappingURL=Load.d.ts.map