var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../core/ExpressionParser", "../core/types", "../services/NavigationService", "./baseTypes"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var ExpressionParser_1 = require("../core/ExpressionParser");
    var types_1 = require("../core/types");
    var NavigationService_1 = require("../services/NavigationService");
    var baseTypes_1 = require("./baseTypes");
    /**
     * Loads given method on based on init/watch properties.
     * If init is true, method will be executed when view model is initialized.
     * If watch is true, method will be executed when any of `this.*.*` properties are
     * modified. This method can be asynchronous. Watch will ignore all assignment
     * changes within the method.
     *
     * Every execution will be delayed by parameter specified in {@link ILoadOptions#watchDelayMS},
     * so multiple calls can be accumulated and only one final execution will proceed. This is useful
     * when you want to load items from API when user is continuously typing in search box.
     *
     * Method will have an input parameter for cancelToken {@link CancelToken} which you
     * can pass it to any REST Api call, before executing next method, cancelToken will
     * cancel previous execution.
     *
     * Either init or watch has to be true, or both can be true as well.
     */
    function Load(_a) {
        var init = _a.init, showErrorOnInit = _a.showErrorOnInit, watch = _a.watch, watchDelayMS = _a.watchDelayMS;
        // tslint:disable-next-line: only-arrow-functions
        return function (target, key) {
            var _this = this;
            baseTypes_1.registerInit(target, function (vm) {
                // tslint:disable-next-line: ban-types
                var oldMethod = vm[key];
                var app = vm.app;
                var showError = init ? (showErrorOnInit ? true : false) : true;
                var ct = new types_1.CancelToken();
                /**
                 * For the special case of init and watch both are true,
                 * we need to make sure that watch is ignored for first run
                 *
                 * So executing is set to true for the first time
                 */
                var executing = init;
                var m = function (ctx) { return __awaiter(_this, void 0, void 0, function () {
                    var ns, pe, e_1;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                ns = app.resolve(NavigationService_1.NavigationService);
                                _a.label = 1;
                            case 1:
                                _a.trys.push([1, 4, 6, 7]);
                                pe = oldMethod.call(vm, ctx);
                                if (!(pe && pe.then)) return [3 /*break*/, 3];
                                return [4 /*yield*/, pe];
                            case 2: return [2 /*return*/, _a.sent()];
                            case 3: return [3 /*break*/, 7];
                            case 4:
                                e_1 = _a.sent();
                                if (/^(cancelled|canceled)$/i.test(e_1.toString().trim())) {
                                    // tslint:disable-next-line: no-console
                                    console.warn(e_1);
                                    return [2 /*return*/];
                                }
                                if (!showError) {
                                    // tslint:disable-next-line: no-console
                                    console.error(e_1);
                                    return [2 /*return*/];
                                }
                                return [4 /*yield*/, ns.alert(e_1, "Error")];
                            case 5:
                                _a.sent();
                                return [3 /*break*/, 7];
                            case 6:
                                showError = true;
                                executing = false;
                                return [7 /*endfinally*/];
                            case 7: return [2 /*return*/];
                        }
                    });
                }); };
                if (watch) {
                    var fx_1 = function (c1) { return __awaiter(_this, void 0, void 0, function () {
                        var ct2, ex1_1;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    if (ct) {
                                        ct.cancel();
                                    }
                                    ct2 = ct = (c1 || new types_1.CancelToken());
                                    if (executing) {
                                        return [2 /*return*/];
                                    }
                                    executing = true;
                                    _a.label = 1;
                                case 1:
                                    _a.trys.push([1, 3, 4, 5]);
                                    return [4 /*yield*/, m(ct2)];
                                case 2:
                                    _a.sent();
                                    return [3 /*break*/, 5];
                                case 3:
                                    ex1_1 = _a.sent();
                                    if (/^(cancelled|canceled)$/i.test(ex1_1.toString().trim())) {
                                        // tslint:disable-next-line: no-console
                                        console.warn(ex1_1);
                                    }
                                    else {
                                        // tslint:disable-next-line: no-console
                                        console.error(ex1_1);
                                    }
                                    return [3 /*break*/, 5];
                                case 4:
                                    executing = false;
                                    ct = null;
                                    return [7 /*endfinally*/];
                                case 5: return [2 /*return*/];
                            }
                        });
                    }); };
                    var timeout_1 = null;
                    // get path stripped as we are passing CancelToken, it will not
                    // parse for this. expressions..
                    var pathList = ExpressionParser_1.parsePath(oldMethod.toString(), true);
                    if (pathList.length === 0) {
                        throw new Error("Nothing to watch !!");
                    }
                    vm.setupWatch(pathList, function () {
                        if (executing) {
                            return;
                        }
                        if (timeout_1) {
                            clearTimeout(timeout_1);
                        }
                        timeout_1 = setTimeout(function () {
                            timeout_1 = null;
                            fx_1();
                        }, watchDelayMS || 100);
                    });
                    vm[key] = fx_1;
                }
                if (init) {
                    app.runAsync(function () { return m.call(vm, ct); });
                }
            });
        };
    }
    exports.default = Load;
});
//# sourceMappingURL=Load.js.map